function [SR, report] = calmsr(LRImages, model, varargin)
    
    % Default parameters
    calmsrParams = getCALMSROptimizationParams;
    if nargin > 2
        calmsrParams = varargin{1};
    end
    
    % Assemble low-resolution observation vector.
    numFrames = size(LRImages, 3);
    y = LRImages(:,:,1:numFrames); 
    y = y(:);
    
    % Initialize the high-resolution image.
    if isempty(model.SR)
        model.SR = imresize( medfilttemp(LRImages, model.motionParams), model.magFactor );
    end
    
    % Assemble static parameters of the equation system for joint motion 
    % estimation and super-resolution.
    optStruct = assembleOptimizationStruct(LRImages, model);
    
    % Transform motion parameters given as image-to-image homography to 
    % parameter vector.
    model.motionParams = motionToParameterVector(model.motionParams, numFrames);
    theta = model.motionParams;
    theta = theta(:,2:numFrames);
    theta = theta(:);
    
    % Initialize the confidence weights of the observation model.
    observationConfidenceWeights = ones(numel(LRImages), 1);
    
    % Initialize the prior weights.
    priorConfidenceWeights = ones( ((2*calmsrParams.btvWindowSize + 1)^2 - 1) * numel(model.SR), 1);
    [x, priorConfidenceWeights] = getInitialHRImage(model.SR(:), y, model, priorConfidenceWeights, optStruct, calmsrParams);
    
    % Setup report structure.
    report.SR{1} = reshape(x, optStruct.Size(1), optStruct.Size(2));    
    report.motion{1} = theta;
    
    % Joint estimation for motion parameters and high-resolution image.
    for iter = 1 : calmsrParams.maxLMIter
        
        % Compute updates for the motion parameters and the high-resolution 
        % image.
        if isequal(calmsrParams.algorithm, 'GN')         
            % Gauss-Newton (GN) iteration
            [A, b, priorConfidenceWeights, observationConfidenceWeights] = assembleEquationSystem(x, optStruct, model, calmsrParams, 0, priorConfidenceWeights, observationConfidenceWeights);
            [delta, ~] = pcg(A, b, calmsrParams.terminationTol, calmsrParams.maxCGIter);
        elseif isequal(calmsrParams.algorithm, 'LM')        
            % Levenberg-Marquardt (LM) iteration 
            [~, delta, priorConfidenceWeights, observationConfidenceWeights] = mu_search(x, theta, model, optStruct, calmsrParams, priorConfidenceWeights, observationConfidenceWeights);
        end
        
        % Update motion parameters and high-resolution image.        
        x_old = x;
        [theta, x] = updateMotionParametersAndHRImage(theta, x_old, delta, numFrames);
        % Record current estimate of the motion parameters.
        motionParams(:,1) = [1; 0; 0; 0];
        motionParams(:,2:numFrames) =  reshape(theta, 4, numFrames-1);
        model.motionParams = motionParams;
            
        % Update report structure.
        report.SR{iter+1} = reshape(x, optStruct.Size(1), optStruct.Size(2));        
        report.motion{iter+1} = model.motionParams;
        
        % Check for convergence.
        norm_dif = max(abs(x - x_old));
        disp(['Step ' num2str(iter) '/' num2str(calmsrParams.maxLMIter) ': ' num2str(norm_dif)]);
        if norm_dif < calmsrParams.terminationTol
            disp('Stop iterations because convergence was reached');
            SR = reshape(x, optStruct.Size(1), optStruct.Size(2));
            return;
        end
       
    end
   
    SR = reshape(x, optStruct.Size(1), optStruct.Size(2));
    
end

function [x, priorConfidenceWeights] = getInitialHRImage(x0, y, model, priorConfidenceWeights, optStruct, calmsrParams)
    
    % Assemble super-resolution system matrix.
    [W, ~] = getSystemMatrixAndJacobian(x0, optStruct, model.motionParams, model.magFactor);
    % Assemble regularization term in matrix notation.
    [T, priorConfidenceWeights] = getRegularizationMatrix(calmsrParams.imagePrior, [], optStruct.Size(1), optStruct.Size(2), priorConfidenceWeights, calmsrParams);
    
    % Assemble linear equation system.
    A0 = W'*W + calmsrParams.imageRegularizationWeight * T;
    b0 = W'*y;
    
    % Determine initial solution by means of CG iterations.
    [x, ~] = pcg(A0, b0, calmsrParams.terminationTol, calmsrParams.maxCGIter);
    
end

function [theta, x] = updateMotionParametersAndHRImage(theta_old, x_old, delta, numFrames)

    % Extract updates from the joint parameter vector.
    deltaA = delta(1:4*(numFrames-1));
    deltaX = delta(4*(numFrames-1)+1 : size(delta));
    % Update both parameter sets.
    x = x_old + deltaX;             
    theta = theta_old + deltaA;

end

function [mu, x, priorConfidenceWeights, observationConfidenceWeights] = mu_search(x, theta, model, optStruct, calmsrParams, priorConfidenceWeights, observationConfidenceWeights)
    
    % Find the optimal damping parameter.
    muSearchRange = calmsrParams.dampingParameterSearchRange;
    [x_, energyFunVals, priorConfidenceWeights, observationConfidenceWeights] =  mu_search_(x, theta, model, optStruct, calmsrParams, priorConfidenceWeights, observationConfidenceWeights);
    [~, minPos] = min(energyFunVals);
    mu = muSearchRange(minPos);
    
    % Select x,r values
    x = x_{minPos};
    priorConfidenceWeights = priorConfidenceWeights{minPos};
    observationConfidenceWeights = observationConfidenceWeights{minPos};
    
end

function [delta, energyFunVals, priorConfidenceWeightsEst, observationConfidenceWeightsEst] = mu_search_(x, theta_cur, model, optStruct, calmsrParams, priorConfidenceWeights, observationConfidenceWeights)
    
    energyFunVals = zeros(length(calmsrParams.dampingParameterSearchRange)); 
    for iter = 1:length(calmsrParams.dampingParameterSearchRange)   
        % Solve with current damping parameter.
        [A, b, priorConfidenceWeightsEst{iter}, observationConfidenceWeightsEst{iter}] = assembleEquationSystem(x, optStruct, model, calmsrParams, calmsrParams.dampingParameterSearchRange(iter), priorConfidenceWeights, observationConfidenceWeights);
        [delta{iter}, ~] = pcg(A, b, calmsrParams.terminationTol, calmsrParams.maxCGIter);
        % Record the value of the energy function associated with the
        % current damping parameter.
        energyFunVals(iter) = get_objective_function(x, theta_cur, delta{iter}, optStruct, model, observationConfidenceWeightsEst{iter});   
    end
    
end

function f_x = get_objective_function(x_old, theta_cur, delta, optStruct, model, observationConfidenceWeights)
    
    LRImages =  optStruct.LRImages;
    numFrames = optStruct.numFrames;
    if ~isequal(delta, 0)
        deltaA = delta(1:4*(numFrames-1));
        deltaX = delta(4*(numFrames-1)+1: size(delta));
        % New image estimation.
        x = x_old + deltaX;
        % New motion estimation
        theta = theta_cur + deltaA;
        % transform new alpha
        alpha_resh = reshape(theta, 4, numFrames-1);
        alpha_new(:,1) = [1; 0; 0; 0];
        alpha_new(:,2:numFrames) =  alpha_resh;
        model.motionParams = alpha_new;
    else
        x = x_old;
    end

    [W, ~] = getSystemMatrixAndJacobian(x, optStruct, model.motionParams, model.magFactor);
    g = LRImages(:);
    r = g - W * x;
    f_x = sum(observationConfidenceWeights .* r.^2);
    
end