%% Setup Matlab path.
addpath('../../');
setupSRToolbox;

addpath('IAT_v0.9.1');
iat_setup;

%% Load low-resolution frames
load('test_chart.mat');

% Setup super-resolution model parameters
model = SRModel;
model.magFactor = 5;
model.psfWidth = 0.25;
transform = 'translation';
refFrame = 8;

% Motion estimation for individual low-resolution frames.
if isempty(model.motionParams)
    % Set parameters for ECC registration algorithm.
    eccParams.iterations = 30;
    eccParams.levels = 2;
    eccParams.transform = transform;
    for m = 1:size(LRImages, 3)
        
        if m == refFrame
            motionParams{m} = eye(3,3);
            continue;
        end
        
        % Pair-wise registration of the current frame to the reference.
        I = LRImages(:,:,refFrame);
        H = iat_ecc(LRImages(:,:,m), I, eccParams);
        
        % Assemble motion parameter structure.
        if strcmp(transform, 'translation')
            H = [1 0 H(1); 0 1 H(2); 0 0 1];
        else
            if size(H, 1) < 3
                H = [H; 0 0 1];
            end
        end
        motionParams{m} = inv(H);
    end
    model.motionParams = motionParams;
end

%% Interpolation of the low-resolution data.
ROI = [10 10 20 20];
ROI_sr = model.magFactor * ROI;
LR_nn = imresize(LRImages(:,:,1), model.magFactor, 'nearest');
figure; 
imshow(LR_nn);

%% Super-resolution using iteratively re-weighted minimization.
% Setup weighted bilateral total variation prior.
model.imagePrior = SRPrior('function', @btvPriorWeighted, 'gradient', @btvPriorWeighted_grad, 'weight', [], 'parameters', {model.magFactor * size(LRImages(:,:,1)), 2, 0.5, []});
% Apply super-resolution
reweightedOptimParams = getReweightedOptimizationParams;
[SR_irwsr, model_irwsr, report] = superresolve(LRImages, model, 'reweightedSR', reweightedOptimParams);

figure; 
imshow(uint8(255*SR_irwsr));

%% Super-resolution using L2 norm + Tikhonov regularization
h = [1 1 1;
     1 -8 1;
     1  1 1];
model.imagePrior = SRPrior('function', @gaussianPrior, 'gradient', @gaussianPrior_grad, 'weight', 1e-2, 'parameters', {model.magFactor * size(LRImages(:,:,1)), h});
model.errorModel = 'l2NormErrorModel';
SR_l2 = superresolve(LRImages, model, 'map');

figure; 
imshow(uint8(255*SR_l2));

%% Visualize observation confidence maps.
iter = [1 10];
frame = [11];
M = numel(LRImages(:,:,1));
for m = 1:length(frame)
    for t = 1:length(iter)
        beta = report.observationWeights{iter(t), frame(m)};
        beta = vectorToImage(beta(1:M), size(LRImages));
        figure; imshow(beta);
    end
end

%% Visualize image prior confidence.
alpha0 = 0.5;
P = 2;
for t = 1:length(iter)
    N = numel(report.SR{iter(t)});
    a = report.priorWeights{iter(t)};
    
    k = 0;
    alpha_norm = 0;
    alpha = 0;
    for l = -P:P
        for m = -P:P    
            if (l ~= 0) || (m ~= 0)
                k = k + 1;
                alpha = alpha + alpha0^(abs(l) + abs(m)) * vectorToImage( a(((k-1)*N + 1):(k*N)), size(report.SR{iter(t)}));
                alpha_norm = alpha_norm + alpha0^(abs(l) + abs(m));
            end 
        end
    end
    figure; imshow(alpha / alpha_norm);
end
