%% Setup all paths    
addpath('../../algorithms');
addpath('../../algorithms/MAP');
addpath('../../algorithms/IRLS');
addpath('../../algorithms/Priors');
addpath('../../common');
addpath('../../common/3rdparty/netlab3_3');

%% Load low-resolution input data
% Chose the number of frames
I = readRangeSequence('Data/outlierSequence/Range', '*.txt');
% Subsampling to desired resolution if required
for k = 1:size(I,3)
    I_range(:,:,k) = imresize(I(:,:,k), [48 64], 'method', 'nearest');
end
I_RGB = readImageSequence('Data/outlierSequence/RGB', '*.png');

% Data normalization
I_range = (I_range - min(I_range(:))) ./ (max(I_range(:)) - min(I_range(:)));

%% Motion estimation
% Use optical flow to estimate subpixel displacements.
ofl_param = [0.025, ...  % Regularization weight
             0.5, ...    % Downsample ratio
             20, ...     % Width of image pyramid coarsest level
             7, ...      % Number of outer fixed point iterations
             1, ...      % Number of inner fixed point iterations
             20];        % Number of SOR iterations
 
 % Choose central frame as reference.
 numFrames = size(I_range, 3);
 refFrameIdx = round((numFrames+1)/2); 
 for k = 1:numFrames
    if k == refFrameIdx
        % Set displacements to zero for the reference frame.
        motionParams{k}.vx = zeros(size(I_RGB(:,:,1)));
        motionParams{k}.vy = zeros(size(I_RGB(:,:,1)));
        I_RGB_warped(:,:,k) = rgb2gray(im2double(I_RGB(:,:,:,refFrameIdx)));
        I_RGB_fixed(:,:,k) = rgb2gray(im2double(I_RGB(:,:,:,refFrameIdx)));
    else
        % Estimate subpixel displacements FROM current frame TO the
        % reference frame using coarse-to-fine optical flow.
        [motionParams{k}.vx, motionParams{k}.vy, Iref_warped] = Coarse2FineTwoFrames(im2double(I_RGB(:,:,:,k)), im2double(I_RGB(:,:,:,refFrameIdx)), ofl_param);
        I_RGB_warped(:,:,k) = rgb2gray(Iref_warped);
        I_RGB_fixed(:,:,k) = rgb2gray(im2double(I_RGB(:,:,:,k)));  
    end
end


%% Super-resolution reconstruction
% Setup model parameters
model = SRModel;

% Set the desired magnification factor.
model.magFactor = 2;
model.psfWidth = 0.2;
% Set motion parameters used for super-resolution.
model.motionParams = motionParams;

% Setup regularization using Huber prior
lambda = 15;    % Regularization weight
alpha = 5e-3;   % Huber threshold
D = getLaplacianMatrix(model.magFactor * size(I_range(:,:,1)));
model.imagePrior = SRPrior('function', @huberPrior, 'gradient', @huberPrior_grad, 'weight', lambda, 'parameters', {D, alpha});

% Error model for low-resolution images used for image reconstruction. We
% use least square estimation in this example.
model.errorModel = 'l2NormErrorModel';

% Set initial guess for super-resolved image. For this example, the bicubic
% upsampled reference frame is used as initial guess.
I_bc = imresize(I_range(:,:,refFrameIdx), model.magFactor);
model.SR = I_bc;

% Multi-sensor super-resolution based on unweighted L2 minimization
solverParams = SRSolverParams('maxFunEvals', 5, 'maxIter', 5, 'tolX', 1e-3, 'tolF', 1e-3, 'verbose', true);
[I_sr, model] = superresolve(I_range, model, 'multisens', solverParams);

% Robust multi-sensor super-resolution using outlier detection on
% photometric data using image similarity and on range data using
% iteratively re-weighted L2 minimization.
model.confidence = detectOutlier(I_RGB_warped, I_RGB_fixed, refFrameIdx, 0.8, size(I_RGB_warped(:,:,1),1) / size(I_range(:,:,1),1), 'patch', size(I_range(:,:,refFrameIdx)));
model.SR = I_sr;
I_sr_robust = superresolve(I_range, model, 'irls', @irlsWeightFunctionHuber, solverParams);

figure; imshow([I_bc I_sr I_sr_robust]);